@@ -6,12 +6,12 @@
 mod utilities;
 
 use cortex_m_rt::entry;
-use stm32h7xx_hal::rcc::{rec, rec::I2c123ClkSel, ResetEnable};
+use stm32h7xx_hal::rcc::{rec, rec::Spi123ClkSel, ResetEnable};
 use stm32h7xx_hal::{pac, prelude::*};
 
 fn enable_fdcan(rec: rec::Fdcan) {
     // Enable and set individual kernel clock to PLL1 Q CK
-    rec.enable().kernel_clk_mux(rec::FdcanClkSel::PLL1_Q);
+    let _ = rec.enable().kernel_clk_mux(rec::FdcanClkSel::PLL1_Q);
 
     // rec is dropped here, and can never be changed again
 }
@@ -28,14 +28,14 @@ fn main() -> ! {
     // Constrain and Freeze clock
     let rcc = dp.RCC.constrain();
     let mut ccdr = rcc
-        .sys_ck(100.mhz())
-        .pll1_q_ck(4.mhz())
-        .pll3_p_ck(4.mhz())
-        .pll3_r_ck(4.mhz())
+        .sys_ck(100.MHz())
+        .pll1_q_ck(4.MHz())
+        .pll3_p_ck(4.MHz())
+        .pll3_r_ck(4.MHz())
         .freeze(pwrcfg, &dp.SYSCFG);
 
-    // Set group kernel clock to PLL3 R CK. Needs mutable ccdr
-    ccdr.peripheral.kernel_i2c123_clk_mux(I2c123ClkSel::PLL3_R);
+    // Set group kernel clock to PLL3 P CK. Needs mutable ccdr
+    ccdr.peripheral.kernel_spi123_clk_mux(Spi123ClkSel::PLL3_P);
 
     // (now ccdr can be immutable)
 
