@@ -23,19 +23,19 @@ fn main() -> ! {
     // Constrain and Freeze clock
     info!("Setup RCC...                  ");
     let rcc = dp.RCC.constrain();
-    let ccdr = rcc.sys_ck(100.mhz()).freeze(pwrcfg, &dp.SYSCFG);
+    let ccdr = rcc.sys_ck(100.MHz()).freeze(pwrcfg, &dp.SYSCFG);
 
     info!("");
     info!("stm32h7xx-hal example - RCC");
     info!("");
 
     // HCLK
-    info!("hclk = {} MHz", ccdr.clocks.hclk().0 as f32 / 1e6);
-    assert_eq!(ccdr.clocks.hclk().0, 50_000_000);
+    info!("hclk = {} MHz", ccdr.clocks.hclk().raw() as f32 / 1e6);
+    assert_eq!(ccdr.clocks.hclk().raw(), 50_000_000);
 
     // SYS_CK
-    info!("sys_ck = {} MHz", ccdr.clocks.sys_ck().0 as f32 / 1e6);
-    assert_eq!(ccdr.clocks.sys_ck().0, 100_000_000);
+    info!("sys_ck = {} MHz", ccdr.clocks.sys_ck().raw() as f32 / 1e6);
+    assert_eq!(ccdr.clocks.sys_ck().raw(), 100_000_000);
 
     loop {
         cortex_m::asm::nop()
