@@ -22,7 +22,7 @@ fn main() -> ! {
     // Constrain and Freeze clock
     info!("Setup RCC...                  ");
     let rcc = dp.RCC.constrain();
-    let ccdr = rcc.sys_ck(96.mhz()).freeze(pwrcfg, &dp.SYSCFG);
+    let ccdr = rcc.sys_ck(96.MHz()).freeze(pwrcfg, &dp.SYSCFG);
 
     #[cfg(any(feature = "rm0433", feature = "rm0455"))]
     let mut watchdog = SystemWindowWatchdog::new(dp.WWDG, &ccdr);
@@ -33,6 +33,10 @@ fn main() -> ! {
     #[cfg(all(feature = "rm0399", feature = "cm4"))]
     let mut watchdog = SystemWindowWatchdog::new(dp.WWDG2, &ccdr);
 
+    // RM0468
+    #[cfg(all(feature = "rm0468"))]
+    let mut watchdog = SystemWindowWatchdog::new(dp.WWDG1, &ccdr);
+
     info!("");
     info!("stm32h7xx-hal example - Watchdog");
     info!("");
@@ -43,7 +47,7 @@ fn main() -> ! {
 
     // Enable the watchdog with a limit of 100 ms and wait forever
     // -> restart the chip
-    watchdog.start(100.ms());
+    watchdog.start(100.millis());
 
     loop {
         cortex_m::asm::nop()
