@@ -2,31 +2,32 @@
 //!
 //! ADC1 and ADC2 share a reset line. To initialise both of them, use the
 //! [`adc12`](adc12) method.
+//!
+//! # Examples
+//!
+//! - [Reading a voltage using ADC1](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/adc.rs)
+//! - [Reading a temperature using ADC3](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/temperature.rs)
+//! - [Using ADC1 and ADC2 together](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/adc12.rs)
+//! - [Using ADC1 and ADC2 in parallel](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/adc12_parallel.rs)
 
 use crate::hal::adc::{Channel, OneShot};
 use crate::hal::blocking::delay::DelayUs;
 
+use core::convert::Infallible;
 use core::marker::PhantomData;
 
+use nb::block;
+
 #[cfg(feature = "rm0455")]
 use crate::stm32::ADC12_COMMON;
 use crate::stm32::{ADC1, ADC2};
 #[cfg(not(feature = "rm0455"))]
 use crate::stm32::{ADC3, ADC3_COMMON};
 
-use crate::gpio::gpioa::{PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7};
-use crate::gpio::gpiob::{PB0, PB1};
-use crate::gpio::gpioc::{PC0, PC1, PC2, PC3, PC4, PC5};
-#[cfg(not(feature = "rm0455"))]
-use crate::gpio::gpiof::{PF10, PF3, PF4, PF5, PF6, PF7, PF8, PF9};
-use crate::gpio::gpiof::{PF11, PF12, PF13, PF14};
-#[cfg(not(feature = "rm0455"))]
-use crate::gpio::gpioh::{PH2, PH3, PH4, PH5};
-use crate::gpio::Analog;
+use crate::gpio::{self, Analog};
 use crate::rcc::rec::AdcClkSelGetter;
 use crate::rcc::{rec, CoreClocks, ResetEnable};
 use crate::time::Hertz;
-use stm32h7::Variant::Val;
 
 #[cfg(not(feature = "revision_v"))]
 const ADC_KER_CK_MAX: u32 = 36_000_000;
@@ -34,9 +35,9 @@ const ADC_KER_CK_MAX: u32 = 36_000_000;
 #[cfg(feature = "revision_v")]
 const ADC_KER_CK_MAX: u32 = 100_000_000;
 
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 pub type Resolution = crate::stm32::adc3::cfgr::RES_A;
-#[cfg(feature = "rm0455")]
+#[cfg(any(feature = "rm0455", feature = "rm0468"))]
 pub type Resolution = crate::stm32::adc1::cfgr::RES_A;
 
 trait NumberOfBits {
@@ -69,6 +70,7 @@ pub struct Adc<ADC, ED> {
     sample_time: AdcSampleTime,
     resolution: Resolution,
     lshift: AdcLshift,
+    current_channel: Option<u8>,
     _enabled: PhantomData<ED>,
 }
 
@@ -78,6 +80,7 @@ pub struct Adc<ADC, ED> {
 //
 // Refer to RM0433 Rev 6 - Chapter 24.4.13
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[allow(non_camel_case_types)]
 pub enum AdcSampleTime {
     /// 1.5 cycles sampling time
@@ -124,6 +127,7 @@ impl From<AdcSampleTime> for u8 {
 ///
 /// Only values in range of 0..=15 are allowed.
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct AdcLshift(u8);
 
 impl AdcLshift {
@@ -145,6 +149,7 @@ impl AdcLshift {
 }
 
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct AdcCalOffset(u16);
 
 impl AdcCalOffset {
@@ -154,6 +159,7 @@ impl AdcCalOffset {
 }
 
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct AdcCalLinear([u32; 6]);
 
 impl AdcCalLinear {
@@ -222,66 +228,66 @@ pub struct Temperature;
 //
 // Refer to DS12110 Rev 7 - Chapter 5 (Table 9)
 adc_pins!(ADC1,
-          // 0, 1 are Pxy_C pins
-          PF11<Analog> => 2,
-          PA6<Analog> => 3,
-          PC4<Analog> => 4,
-          PB1<Analog> => 5,
-          PF12<Analog> => 6,
-          PA7<Analog> => 7,
-          PC5<Analog> => 8,
-          PB0<Analog> => 9,
-          PC0<Analog> => 10,
-          PC1<Analog> => 11,
-          PC2<Analog> => 12,
-          PC3<Analog> => 13,
-          PA2<Analog> => 14,
-          PA3<Analog> => 15,
-          PA0<Analog> => 16,
-          PA1<Analog> => 17,
-          PA4<Analog> => 18,
-          PA5<Analog> => 19,
+    // 0, 1 are Pxy_C pins
+    gpio::PF11<Analog> => 2,
+    gpio::PA6<Analog> => 3,
+    gpio::PC4<Analog> => 4,
+    gpio::PB1<Analog> => 5,
+    gpio::PF12<Analog> => 6,
+    gpio::PA7<Analog> => 7,
+    gpio::PC5<Analog> => 8,
+    gpio::PB0<Analog> => 9,
+    gpio::PC0<Analog> => 10,
+    gpio::PC1<Analog> => 11,
+    gpio::PC2<Analog> => 12,
+    gpio::PC3<Analog> => 13,
+    gpio::PA2<Analog> => 14,
+    gpio::PA3<Analog> => 15,
+    gpio::PA0<Analog> => 16,
+    gpio::PA1<Analog> => 17,
+    gpio::PA4<Analog> => 18,
+    gpio::PA5<Analog> => 19,
 );
 
 adc_pins!(ADC2,
-          // 0, 1 are Pxy_C pins
-          PF13<Analog> => 2,
-          PA6<Analog> => 3,
-          PC4<Analog> => 4,
-          PB1<Analog> => 5,
-          PF14<Analog> => 6,
-          PA7<Analog> => 7,
-          PC5<Analog> => 8,
-          PB0<Analog> => 9,
-          PC0<Analog> => 10,
-          PC1<Analog> => 11,
-          PC2<Analog> => 12,
-          PC3<Analog> => 13,
-          PA2<Analog> => 14,
-          PA3<Analog> => 15,
-          // 16, 17 are dac_outX
-          PA4<Analog> => 18,
-          PA5<Analog> => 19,
+    // 0, 1 are Pxy_C pins
+    gpio::PF13<Analog> => 2,
+    gpio::PA6<Analog> => 3,
+    gpio::PC4<Analog> => 4,
+    gpio::PB1<Analog> => 5,
+    gpio::PF14<Analog> => 6,
+    gpio::PA7<Analog> => 7,
+    gpio::PC5<Analog> => 8,
+    gpio::PB0<Analog> => 9,
+    gpio::PC0<Analog> => 10,
+    gpio::PC1<Analog> => 11,
+    gpio::PC2<Analog> => 12,
+    gpio::PC3<Analog> => 13,
+    gpio::PA2<Analog> => 14,
+    gpio::PA3<Analog> => 15,
+    // 16, 17 are dac_outX
+    gpio::PA4<Analog> => 18,
+    gpio::PA5<Analog> => 19,
 );
 
 #[cfg(not(feature = "rm0455"))]
 adc_pins!(ADC3,
-          // 0, 1 are Pxy_C pins
-          PF9<Analog> => 2,
-          PF7<Analog> => 3,
-          PF5<Analog> => 4,
-          PF3<Analog> => 5,
-          PF10<Analog> => 6,
-          PF8<Analog> => 7,
-          PF6<Analog> => 8,
-          PF4<Analog> => 9,
-          PC0<Analog> => 10,
-          PC1<Analog> => 11,
-          PC2<Analog> => 12,
-          PH2<Analog> => 13,
-          PH3<Analog> => 14,
-          PH4<Analog> => 15,
-          PH5<Analog> => 16,
+    // 0, 1 are Pxy_C pins
+    gpio::PF9<Analog> => 2,
+    gpio::PF7<Analog> => 3,
+    gpio::PF5<Analog> => 4,
+    gpio::PF3<Analog> => 5,
+    gpio::PF10<Analog> => 6,
+    gpio::PF8<Analog> => 7,
+    gpio::PF6<Analog> => 8,
+    gpio::PF4<Analog> => 9,
+    gpio::PC0<Analog> => 10,
+    gpio::PC1<Analog> => 11,
+    gpio::PC2<Analog> => 12,
+    gpio::PH2<Analog> => 13,
+    gpio::PH3<Analog> => 14,
+    gpio::PH4<Analog> => 15,
+    gpio::PH5<Analog> => 16,
 );
 #[cfg(not(feature = "rm0455"))]
 adc_internal!(
@@ -316,20 +322,38 @@ pub trait AdcExt<ADC>: Sized {
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub struct StoredConfig(AdcSampleTime, Resolution, AdcLshift);
 
+#[cfg(feature = "defmt")]
+impl defmt::Format for StoredConfig {
+    fn format(&self, fmt: defmt::Formatter) {
+        defmt::write!(
+            fmt,
+            "StoredConfig({:?}, {:?}, {:?})",
+            self.0,
+            defmt::Debug2Format(&self.1),
+            self.2
+        )
+    }
+}
+
 /// Get and check the adc_ker_ck_input
 fn check_clock(prec: &impl AdcClkSelGetter, clocks: &CoreClocks) -> Hertz {
     // Select Kernel Clock
     let adc_clock = match prec.get_kernel_clk_mux() {
-        Val(rec::AdcClkSel::PLL2_P) => clocks.pll2_p_ck(),
-        Val(rec::AdcClkSel::PLL3_R) => clocks.pll3_r_ck(),
-        Val(rec::AdcClkSel::PER) => clocks.per_ck(),
+        Some(rec::AdcClkSel::PLL2_P) => {
+            clocks.pll2_p_ck().expect("ADC: PLL2_P must be enabled")
+        }
+        Some(rec::AdcClkSel::PLL3_R) => {
+            clocks.pll3_r_ck().expect("ADC: PLL3_R must be enabled")
+        }
+        Some(rec::AdcClkSel::PER) => {
+            clocks.per_ck().expect("ADC: PER clock must be enabled")
+        }
         _ => unreachable!(),
-    }
-    .expect("adc_ker_ck_input is not running!");
+    };
 
     // Check against datasheet requirements
     assert!(
-        adc_clock.0 <= ADC_KER_CK_MAX,
+        adc_clock.raw() <= ADC_KER_CK_MAX,
         "adc_ker_ck_input is too fast"
     );
 
@@ -365,7 +389,7 @@ pub fn adc12(
     adc2.power_down();
 
     // Reset peripheral
-    prec.reset();
+    let _ = prec.reset(); // drop, can be recreated by free method
 
     // Power Up, Preconfigure and Calibrate
     adc1.power_up(delay);
@@ -451,7 +475,7 @@ macro_rules! adc_hal {
                     adc.power_down();
 
                     // Reset peripheral
-                    prec.reset();
+                    let _ = prec.reset(); // drop, can be recreated by free method
 
                     // Power Up, Preconfigure and Calibrate
                     adc.power_up(delay);
@@ -467,6 +491,7 @@ macro_rules! adc_hal {
                         sample_time: AdcSampleTime::default(),
                         resolution: Resolution::SIXTEENBIT,
                         lshift: AdcLshift::default(),
+                        current_channel: None,
                         _enabled: PhantomData,
                     }
                 }
@@ -568,6 +593,7 @@ macro_rules! adc_hal {
                         sample_time: self.sample_time,
                         resolution: self.resolution,
                         lshift: self.lshift,
+                        current_channel: None,
                         _enabled: PhantomData,
                     }
                 }
@@ -610,9 +636,17 @@ macro_rules! adc_hal {
                     }
                 }
 
+                /// Start conversion
+                ///
+                /// This method will start reading sequence on the given pin.
+                /// The value can be then read through the `read_sample` method.
                 // Refer to RM0433 Rev 6 - Chapter 24.4.16
-                fn convert(&mut self, chan: u8) -> u32 {
+                pub fn start_conversion<PIN>(&mut self, _pin: &mut PIN)
+                    where PIN: Channel<$ADC, ID = u8>,
+                {
+                    let chan = PIN::channel();
                     assert!(chan <= 19);
+
                     self.check_conversion_conditions();
 
                     // Set resolution
@@ -628,19 +662,32 @@ macro_rules! adc_hal {
                         w.sq1().bits(chan)
                             .l().bits(0)
                     });
+                    self.current_channel = Some(chan);
 
                     // Perform conversion
                     self.rb.cr.modify(|_, w| w.adstart().set_bit());
+                }
 
-                    // Wait until conversion finished
-                    while self.rb.isr.read().eoc().bit_is_clear() {}
+                /// Read sample
+                ///
+                /// `nb::Error::WouldBlock` in case the conversion is still
+                /// progressing.
+                // Refer to RM0433 Rev 6 - Chapter 24.4.16
+                pub fn read_sample(&mut self) -> nb::Result<u32, Infallible> {
+                    let chan = self.current_channel.expect("No channel was selected, use start_conversion first");
+
+                    // Check if the conversion is finished
+                    if self.rb.isr.read().eoc().bit_is_clear() {
+                        return Err(nb::Error::WouldBlock);
+                    }
 
                     // Disable preselection of this channel, refer to RM0433 Rev 6 - Chapter 24.4.12
                     self.rb.pcsel.modify(|r, w| unsafe { w.pcsel().bits(r.pcsel().bits() & !(1 << chan)) });
+                    self.current_channel = None;
 
                     // Retrieve result
                     let result = self.rb.dr.read().bits();
-                    result
+                    nb::Result::Ok(result)
                 }
 
                 fn check_conversion_conditions(&self) {
@@ -678,6 +725,7 @@ macro_rules! adc_hal {
                         sample_time: self.sample_time,
                         resolution: self.resolution,
                         lshift: self.lshift,
+                        current_channel: None,
                         _enabled: PhantomData,
                     }
                 }
@@ -739,12 +787,32 @@ macro_rules! adc_hal {
                     self.lshift = lshift;
                 }
 
-                /// Returns the largest possible sample value for the current settings
+                /// Returns the largest possible sample value for the current ADC configuration
+                ///
+                /// Using this value as the denominator when calculating
+                /// transfer functions results in a gain error, and thus should
+                /// be avoided. Use the [slope](#method.slope) method instead.
+                #[deprecated(since = "0.12.0", note = "See the slope() method instead")]
                 pub fn max_sample(&self) -> u32 {
                     ((1 << self.get_resolution().number_of_bits() as u32) - 1) << self.get_lshift().value() as u32
                 }
 
-                                /// Returns the offset calibration value for single ended channel
+                /// Returns the slope for the current ADC configuration. 1 LSB = Vref / slope
+                ///
+                /// This value can be used in calcuations involving the transfer function of
+                /// the ADC. For example, to calculate an estimate for the
+                /// applied voltage of an ADC channel referenced to voltage
+                /// `vref`
+                ///
+                /// ```
+                /// let v = adc.read(&ch).unwrap() as f32 * vref / adc.slope() as f32;
+                /// ```
+                pub fn slope(&self) -> u32 {
+                    1 << (self.get_resolution().number_of_bits() as u32 + self.get_lshift().value() as u32)
+                }
+
+
+                /// Returns the offset calibration value for single ended channel
                 pub fn read_offset_calibration_value(&self) -> AdcCalOffset {
                     AdcCalOffset(self.rb.calfact.read().calfact_s().bits())
                 }
@@ -802,6 +870,16 @@ macro_rules! adc_hal {
                         panic!("Cannot read linear calibration value when the ADC is disabled");
                     }
                 }
+
+                /// Returns a reference to the inner peripheral
+                pub fn inner(&self) -> &$ADC {
+                    &self.rb
+                }
+
+                /// Returns a mutable reference to the inner peripheral
+                pub fn inner_mut(&mut self) -> &mut $ADC {
+                    &mut self.rb
+                }
             }
 
             impl<WORD, PIN> OneShot<$ADC, WORD, PIN> for Adc<$ADC, Enabled>
@@ -811,8 +889,9 @@ macro_rules! adc_hal {
             {
                 type Error = ();
 
-                fn read(&mut self, _pin: &mut PIN) -> nb::Result<WORD, Self::Error> {
-                    let res = self.convert(PIN::channel());
+                fn read(&mut self, pin: &mut PIN) -> nb::Result<WORD, Self::Error> {
+                    self.start_conversion(pin);
+                    let res = block!(self.read_sample()).unwrap();
                     Ok(res.into())
                 }
             }
