@@ -10,9 +10,10 @@ use super::{
 use core::marker::PhantomData;
 
 use crate::{
+    i2c::I2c,
     pac::{self, DMA1, DMA2, DMAMUX1},
     rcc::{rec, rec::ResetEnable},
-    serial, spi,
+    sai, serial, spi,
 };
 
 use core::ops::Deref;
@@ -72,6 +73,7 @@ impl Instance for DMA2 {
 
 /// DMA interrupts
 #[derive(Debug, Clone, Copy)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct DmaInterrupts {
     transfer_complete: bool,
     transfer_error: bool,
@@ -82,6 +84,7 @@ pub struct DmaInterrupts {
 
 /// Contains configuration for a DMA stream
 #[derive(Debug, Clone, Copy)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct DmaConfig {
     pub(crate) priority: config::Priority,
     pub(crate) memory_increment: bool,
@@ -135,6 +138,7 @@ impl DoubleBufferedConfig for DmaConfig {
 impl DmaConfig {
     /// Set the priority.
     #[inline(always)]
+    #[must_use]
     pub fn priority(mut self, priority: config::Priority) -> Self {
         self.priority = priority;
         self
@@ -142,18 +146,21 @@ impl DmaConfig {
 
     /// Set the memory_increment.
     #[inline(always)]
+    #[must_use]
     pub fn memory_increment(mut self, memory_increment: bool) -> Self {
         self.memory_increment = memory_increment;
         self
     }
     /// Set the peripheral_increment.
     #[inline(always)]
+    #[must_use]
     pub fn peripheral_increment(mut self, peripheral_increment: bool) -> Self {
         self.peripheral_increment = peripheral_increment;
         self
     }
     /// Set the transfer_complete_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn transfer_complete_interrupt(
         mut self,
         transfer_complete_interrupt: bool,
@@ -163,6 +170,7 @@ impl DmaConfig {
     }
     /// Set the half_transfer_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn half_transfer_interrupt(
         mut self,
         half_transfer_interrupt: bool,
@@ -172,6 +180,7 @@ impl DmaConfig {
     }
     /// Set the transfer_error_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn transfer_error_interrupt(
         mut self,
         transfer_error_interrupt: bool,
@@ -181,6 +190,7 @@ impl DmaConfig {
     }
     /// Set the direct_mode_error_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn direct_mode_error_interrupt(
         mut self,
         direct_mode_error_interrupt: bool,
@@ -190,24 +200,28 @@ impl DmaConfig {
     }
     /// Set the fifo_error_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn fifo_error_interrupt(mut self, fifo_error_interrupt: bool) -> Self {
         self.fifo_error_interrupt = fifo_error_interrupt;
         self
     }
     /// Set the circular_buffer.
     #[inline(always)]
+    #[must_use]
     pub fn circular_buffer(mut self, circular_buffer: bool) -> Self {
         self.circular_buffer = circular_buffer;
         self
     }
     /// Set the double_buffer.
     #[inline(always)]
+    #[must_use]
     pub fn double_buffer(mut self, double_buffer: bool) -> Self {
         self.double_buffer = double_buffer;
         self
     }
     /// Set the fifo_threshold.
     #[inline(always)]
+    #[must_use]
     pub fn fifo_threshold(
         mut self,
         fifo_threshold: config::FifoThreshold,
@@ -217,18 +231,21 @@ impl DmaConfig {
     }
     /// Set the fifo_enable.
     #[inline(always)]
+    #[must_use]
     pub fn fifo_enable(mut self, fifo_enable: bool) -> Self {
         self.fifo_enable = fifo_enable;
         self
     }
     /// Set the memory_burst.
     #[inline(always)]
+    #[must_use]
     pub fn memory_burst(mut self, memory_burst: config::BurstMode) -> Self {
         self.memory_burst = memory_burst;
         self
     }
     /// Set the peripheral_burst.
     #[inline(always)]
+    #[must_use]
     pub fn peripheral_burst(
         mut self,
         peripheral_burst: config::BurstMode,
@@ -300,7 +317,7 @@ impl<I: Instance> StreamsTuple<I> {
     /// the other DMA2/1 if is has transfers are enabled. See
     /// <https://github.com/stm32-rs/stm32h7xx-hal/issues/228>
     pub fn new(_regs: I, prec: I::Rec) -> Self {
-        prec.enable();
+        let _ = prec.enable(); // drop
         Self(
             Stream0 { _dma: PhantomData },
             Stream1 { _dma: PhantomData },
@@ -924,13 +941,116 @@ peripheral_target_address!(
 );
 
 peripheral_target_address!(
+    (HAL: I2c<pac::I2C1>, rxdr, u8, P2M, DMAReq::I2C1_RX_DMA),
+    (HAL: I2c<pac::I2C1>, txdr, u8, M2P, DMAReq::I2C1_TX_DMA),
+    (HAL: I2c<pac::I2C2>, rxdr, u8, P2M, DMAReq::I2C2_RX_DMA),
+    (HAL: I2c<pac::I2C2>, txdr, u8, M2P, DMAReq::I2C2_TX_DMA),
+    (HAL: I2c<pac::I2C3>, rxdr, u8, P2M, DMAReq::I2C3_RX_DMA),
+    (HAL: I2c<pac::I2C3>, txdr, u8, M2P, DMAReq::I2C3_TX_DMA),
+);
+
+peripheral_target_address!(
+    // implementation on PAC types, fixed output Channel A and input Channel B
     (pac::SAI1, cha.dr, u32, M2P, DMAReq::SAI1A_DMA),
     (pac::SAI1, chb.dr, u32, P2M, DMAReq::SAI1B_DMA),
+    // implementation on compound types (either Channel A or Channel B)
+    (
+        sai::dma::ChannelA<pac::SAI1>,
+        cha.dr,
+        u32,
+        P2M,
+        DMAReq::SAI1A_DMA
+    ),
+    (
+        sai::dma::ChannelA<pac::SAI1>,
+        cha.dr,
+        u32,
+        M2P,
+        DMAReq::SAI1A_DMA
+    ),
+    (
+        sai::dma::ChannelB<pac::SAI1>,
+        chb.dr,
+        u32,
+        P2M,
+        DMAReq::SAI1B_DMA
+    ),
+    (
+        sai::dma::ChannelB<pac::SAI1>,
+        chb.dr,
+        u32,
+        M2P,
+        DMAReq::SAI1B_DMA
+    ),
+);
+#[cfg(not(feature = "rm0468"))]
+peripheral_target_address!(
+    // implementation on PAC types, fixed output Channel A and input Channel B
     (pac::SAI2, cha.dr, u32, M2P, DMAReq::SAI2A_DMA),
     (pac::SAI2, chb.dr, u32, P2M, DMAReq::SAI2B_DMA),
+    // implementation on compound types (either Channel A or Channel B)
+    (
+        sai::dma::ChannelA<pac::SAI2>,
+        cha.dr,
+        u32,
+        P2M,
+        DMAReq::SAI2A_DMA
+    ),
+    (
+        sai::dma::ChannelA<pac::SAI2>,
+        cha.dr,
+        u32,
+        M2P,
+        DMAReq::SAI2A_DMA
+    ),
+    (
+        sai::dma::ChannelB<pac::SAI2>,
+        chb.dr,
+        u32,
+        P2M,
+        DMAReq::SAI2B_DMA
+    ),
+    (
+        sai::dma::ChannelB<pac::SAI2>,
+        chb.dr,
+        u32,
+        M2P,
+        DMAReq::SAI2B_DMA
+    ),
+
 );
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 peripheral_target_address!(
+    // implementation on PAC types, fixed output Channel A and input Channel B
     (pac::SAI3, cha.dr, u32, M2P, DMAReq::SAI3_A_DMA),
     (pac::SAI3, chb.dr, u32, P2M, DMAReq::SAI3_B_DMA),
+    // implementation on compound types (either Channel A or Channel B)
+    (
+        sai::dma::ChannelA<pac::SAI3>,
+        cha.dr,
+        u32,
+        P2M,
+        DMAReq::SAI3_A_DMA
+    ),
+    (
+        sai::dma::ChannelA<pac::SAI3>,
+        cha.dr,
+        u32,
+        M2P,
+        DMAReq::SAI3_A_DMA
+    ),
+    (
+        sai::dma::ChannelB<pac::SAI3>,
+        chb.dr,
+        u32,
+        P2M,
+        DMAReq::SAI3_B_DMA
+    ),
+    (
+        sai::dma::ChannelB<pac::SAI3>,
+        chb.dr,
+        u32,
+        M2P,
+        DMAReq::SAI3_B_DMA
+    ),
 );
