@@ -44,9 +44,13 @@
 //! specifying that clock. If this results in a configuration that cannot
 //! be achieved by hardware, `freeze` will panic.
 //!
-//! # Configuration Example
+//! # Examples
 //!
-//! A simple example:
+//! - [Simple RCC example](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/rcc.rs).
+//! - [Fractional PLL configuration](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/fractional-pll.rs)
+//! - [MCO example](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/mco.rs)
+//!
+//! Simple example:
 //!
 //! ```rust
 //!     let dp = pac::Peripherals::take().unwrap();
@@ -56,8 +60,8 @@
 //!
 //!     let rcc = dp.RCC.constrain();
 //!     let ccdr = rcc
-//!         .sys_ck(96.mhz())
-//!         .pclk1(48.mhz())
+//!         .sys_ck(96.MHz())
+//!         .pclk1(48.MHz())
 //!         .freeze(pwrcfg, &dp.SYSCFG);
 //! ```
 //!
@@ -71,7 +75,7 @@
 //!
 //!     let rcc = dp.RCC.constrain();
 //!     let ccdr = rcc
-//!         .sys_ck(200.mhz()) // Implies pll1_p_ck
+//!         .sys_ck(200.MHz()) // Implies pll1_p_ck
 //!         // For non-integer values, round up. `freeze` will never
 //!         // configure a clock faster than that specified.
 //!         .pll1_q_ck(33_333_334.hz())
@@ -89,15 +93,15 @@
 //!
 //!     let rcc = dp.RCC.constrain();
 //!     let ccdr = rcc
-//!         .use_hse(25.mhz()) // XTAL X1
-//!         .sys_ck(400.mhz())
-//!         .pll1_r_ck(100.mhz()) // for TRACECK
-//!         .pll1_q_ck(200.mhz())
-//!         .hclk(200.mhz())
+//!         .use_hse(25.MHz()) // XTAL X1
+//!         .sys_ck(400.MHz())
+//!         .pll1_r_ck(100.MHz()) // for TRACECK
+//!         .pll1_q_ck(200.MHz())
+//!         .hclk(200.MHz())
 //!         .pll3_strategy(PllConfigStrategy::Iterative)
-//!         .pll3_p_ck(240.mhz()) // for LTDC
-//!         .pll3_q_ck(48.mhz()) // for LTDC
-//!         .pll3_r_ck(26_666_667.hz()) // Pixel clock for LTDC
+//!         .pll3_p_ck(240.MHz()) // for LTDC
+//!         .pll3_q_ck(48.MHz()) // for LTDC
+//!         .pll3_r_ck(26_666_667.Hz()) // Pixel clock for LTDC
 //!         .freeze(pwrcfg, &dp.SYSCFG);
 //!```
 //!
@@ -113,7 +117,7 @@
 //! let ccdr = ...; // Returned by `freeze()`, see examples above
 //!
 //! // Runtime confirmation that hclk really is 200MHz
-//! assert_eq!(ccdr.clocks.hclk().0, 200_000_000);
+//! assert_eq!(ccdr.clocks.hclk().raw(), 200_000_000);
 //!
 //! // Panics if pll1_q_ck is not running
 //! let _ = ccdr.clocks.pll1_q_ck().unwrap();
@@ -143,11 +147,17 @@ use crate::pwr::PowerConfiguration;
 use crate::pwr::VoltageScale as Voltage;
 use crate::stm32::rcc::cfgr::SW_A as SW;
 use crate::stm32::rcc::cfgr::TIMPRE_A as TIMPRE;
-use crate::stm32::rcc::d1cfgr::HPRE_A as HPRE;
 use crate::stm32::rcc::pllckselr::PLLSRC_A as PLLSRC;
 use crate::stm32::{RCC, SYSCFG};
 use crate::time::Hertz;
 
+#[cfg(feature = "rm0455")]
+use crate::stm32::rcc::cdcfgr1::HPRE_A as HPRE;
+#[cfg(not(feature = "rm0455"))]
+use crate::stm32::rcc::d1cfgr::HPRE_A as HPRE;
+#[cfg(feature = "log")]
+use log::debug;
+
 #[cfg(feature = "rm0455")]
 use crate::stm32::rcc::cdccipr::CKPERSEL_A as CKPERSEL;
 #[cfg(not(feature = "rm0455"))]
@@ -262,11 +272,9 @@ macro_rules! pclk_setter {
         $(
             /// Set the peripheral clock frequency for APB
             /// peripherals.
-            pub fn $name<F>(mut self, freq: F) -> Self
-            where
-                F: Into<Hertz>,
-            {
-                self.config.$pclk = Some(freq.into().0);
+            #[must_use]
+            pub fn $name(mut self, freq: Hertz) -> Self {
+                self.config.$pclk = Some(freq.raw());
                 self
             }
         )+
@@ -279,11 +287,9 @@ macro_rules! pll_setter {
         $(
             $(
                 /// Set the target clock frequency for PLL output
-                pub fn $name<F>(mut self, freq: F) -> Self
-                where
-                    F: Into<Hertz>,
-                {
-                    self.config.$pll.$ck = Some(freq.into().0);
+                #[must_use]
+                pub fn $name(mut self, freq: Hertz) -> Self {
+                    self.config.$pll.$ck = Some(freq.raw());
                     self
                 }
             )+
@@ -297,6 +303,7 @@ macro_rules! pll_strategy_setter {
         $(
             /// Set the PLL divider strategy to be used when the PLL
             /// is configured
+            #[must_use]
             pub fn $name(mut self, strategy: PllConfigStrategy) -> Self
             {
                 self.config.$pll.strategy = strategy;
@@ -310,45 +317,38 @@ impl Rcc {
     /// Uses HSE (external oscillator) instead of HSI (internal RC
     /// oscillator) as the clock source. Will result in a hang if an
     /// external oscillator is not connected or it fails to start.
-    pub fn use_hse<F>(mut self, freq: F) -> Self
-    where
-        F: Into<Hertz>,
-    {
-        self.config.hse = Some(freq.into().0);
+    #[must_use]
+    pub fn use_hse(mut self, freq: Hertz) -> Self {
+        self.config.hse = Some(freq.raw());
         self
     }
 
     /// Use an external clock signal rather than a crystal oscillator,
     /// bypassing the XTAL driver.
+    #[must_use]
     pub fn bypass_hse(mut self) -> Self {
         self.config.bypass_hse = true;
         self
     }
 
     /// Set input frequency to the SCGU
-    pub fn sys_ck<F>(mut self, freq: F) -> Self
-    where
-        F: Into<Hertz>,
-    {
-        self.config.sys_ck = Some(freq.into().0);
+    #[must_use]
+    pub fn sys_ck(mut self, freq: Hertz) -> Self {
+        self.config.sys_ck = Some(freq.raw());
         self
     }
 
     /// Set input frequency to the SCGU - ALIAS
-    pub fn sysclk<F>(mut self, freq: F) -> Self
-    where
-        F: Into<Hertz>,
-    {
-        self.config.sys_ck = Some(freq.into().0);
+    #[must_use]
+    pub fn sysclk(mut self, freq: Hertz) -> Self {
+        self.config.sys_ck = Some(freq.raw());
         self
     }
 
     /// Set peripheral clock frequency
-    pub fn per_ck<F>(mut self, freq: F) -> Self
-    where
-        F: Into<Hertz>,
-    {
-        self.config.per_ck = Some(freq.into().0);
+    #[must_use]
+    pub fn per_ck(mut self, freq: Hertz) -> Self {
+        self.config.per_ck = Some(freq.raw());
         self
     }
 
@@ -356,11 +356,9 @@ impl Rcc {
     /// are several gated versions `rcc_hclk[1-4]` for different power domains,
     /// and the AXI bus clock is called `rcc_aclk`. However they are all the
     /// same frequency.
-    pub fn hclk<F>(mut self, freq: F) -> Self
-    where
-        F: Into<Hertz>,
-    {
-        self.config.rcc_hclk = Some(freq.into().0);
+    #[must_use]
+    pub fn hclk(mut self, freq: Hertz) -> Self {
+        self.config.rcc_hclk = Some(freq.raw());
         self
     }
 
@@ -452,7 +450,7 @@ impl Rcc {
 
         // See RM0433 Rev 7 Table 17. FLASH recommended number of wait
         // states and programming delay
-        #[cfg(not(feature = "rm0455"))]
+        #[cfg(any(feature = "rm0433", feature = "rm0399"))]
         let (wait_states, progr_delay) = match vos {
             // VOS 0 range VCORE 1.26V - 1.40V
             Voltage::Scale0 => match rcc_aclk_mhz {
@@ -535,6 +533,39 @@ impl Rcc {
             },
         };
 
+        // See RM0468 Rev 2 Table 16
+        #[cfg(feature = "rm0468")]
+        let (wait_states, progr_delay) = match vos {
+            // VOS 0 range VCORE 1.25V - 1.35V
+            Voltage::Scale0 => match rcc_aclk_mhz {
+                0..=69 => (0, 0),
+                70..=139 => (1, 1),
+                140..=209 => (2, 2),
+                _ => (3, 3),
+            },
+            // VOS 1 range VCORE 1.15V - 1.25V
+            Voltage::Scale1 => match rcc_aclk_mhz {
+                0..=66 => (0, 0),
+                67..=132 => (1, 1),
+                133..=199 => (2, 2),
+                _ => (3, 3),
+            },
+            // VOS 2 range VCORE 1.05V - 1.15V
+            Voltage::Scale2 => match rcc_aclk_mhz {
+                0..=49 => (0, 0),
+                50..=99 => (1, 1),
+                100..=149 => (2, 2),
+                _ => (3, 3),
+            },
+            // VOS 3 range VCORE 0.95V - 1.05V
+            Voltage::Scale3 => match rcc_aclk_mhz {
+                0..=34 => (0, 0),
+                35..=69 => (1, 1),
+                70..=84 => (2, 2),
+                _ => (3, 3),
+            },
+        };
+
         let flash = unsafe { &(*FLASH::ptr()) };
         // Adjust flash wait states
         flash.acr.write(|w| unsafe {
@@ -567,11 +598,11 @@ impl Rcc {
             };
             self.config.pll1.p_ck = pll1_p_ck;
 
-            (Hertz(sys_ck), true)
+            (Hertz::from_raw(sys_ck), true)
         } else {
             // sys_ck is derived directly from a source clock
             // (HSE/HSI). pll1_p_ck can be as requested
-            (Hertz(sys_ck), false)
+            (Hertz::from_raw(sys_ck), false)
         }
     }
 
@@ -680,7 +711,7 @@ impl Rcc {
         // Set to 1
         let d1cpre_bits = 0;
         let d1cpre_div = 1;
-        let sys_d1cpre_ck = sys_ck.0 / d1cpre_div;
+        let sys_d1cpre_ck = sys_ck.raw() / d1cpre_div;
 
         // Timer prescaler selection
         let timpre = TIMPRE::DEFAULTX2;
@@ -688,7 +719,7 @@ impl Rcc {
         // Refer to part datasheet "General operating conditions"
         // table for (rev V). We do not assert checks for earlier
         // revisions which may have lower limits.
-        #[cfg(not(feature = "rm0455"))]
+        #[cfg(any(feature = "rm0433", feature = "rm0399"))]
         let (sys_d1cpre_ck_max, rcc_hclk_max, pclk_max) = match pwrcfg.vos {
             Voltage::Scale0 => (480_000_000, 240_000_000, 120_000_000),
             Voltage::Scale1 => (400_000_000, 200_000_000, 100_000_000),
@@ -704,6 +735,14 @@ impl Rcc {
             _ => (88_000_000, 88_000_000, 44_000_000),
         };
 
+        #[cfg(feature = "rm0468")] // 725 / 735 / 730
+        let (sys_d1cpre_ck_max, rcc_hclk_max, pclk_max) = match pwrcfg.vos {
+            Voltage::Scale0 => (520_000_000, 275_000_000, 137_500_000),
+            Voltage::Scale1 => (400_000_000, 200_000_000, 100_000_000),
+            Voltage::Scale2 => (300_000_000, 150_000_000, 75_000_000),
+            _ => (170_000_000, 85_000_000, 42_500_000),
+        };
+
         // Check resulting sys_d1cpre_ck
         assert!(sys_d1cpre_ck <= sys_d1cpre_ck_max);
 
@@ -751,7 +790,7 @@ impl Rcc {
             MCO1::HSI => HSI,
             MCO1::LSE => unimplemented!(),
             MCO1::HSE => self.config.hse.unwrap(),
-            MCO1::PLL1_Q => pll1_q_ck.unwrap().0,
+            MCO1::PLL1_Q => pll1_q_ck.unwrap().raw(),
             MCO1::HSI48 => HSI48,
         };
         let (mco_1_pre, mco1_ck) =
@@ -759,10 +798,10 @@ impl Rcc {
 
         let mco2_in = match self.config.mco2.source {
             // We set the required clock earlier, so can unwrap() here.
-            MCO2::SYSCLK => sys_ck.0,
-            MCO2::PLL2_P => pll2_p_ck.unwrap().0,
+            MCO2::SYSCLK => sys_ck.raw(),
+            MCO2::PLL2_P => pll2_p_ck.unwrap().raw(),
             MCO2::HSE => self.config.hse.unwrap(),
-            MCO2::PLL1_P => pll1_p_ck.unwrap().0,
+            MCO2::PLL1_P => pll1_p_ck.unwrap().raw(),
             MCO2::CSI => CSI,
             MCO2::LSI => LSI,
         };
@@ -806,7 +845,7 @@ impl Rcc {
                 });
                 while rcc.cr.read().hserdy().is_not_ready() {}
 
-                Some(Hertz(hse))
+                Some(Hertz::from_raw(hse))
             }
             None => None,
         };
@@ -841,6 +880,7 @@ impl Rcc {
         }
 
         // Core Prescaler / AHB Prescaler / APB3 Prescaler
+        #[cfg(not(feature = "rm0455"))]
         rcc.d1cfgr.modify(|_, w| unsafe {
             w.d1cpre()
                 .bits(d1cpre_bits)
@@ -849,23 +889,49 @@ impl Rcc {
                 .hpre()
                 .variant(hpre_bits)
         });
+        #[cfg(feature = "rm0455")]
+        rcc.cdcfgr1.modify(|_, w| unsafe {
+            w.cdcpre()
+                .bits(d1cpre_bits)
+                .cdppre() // D1/CD contains APB3
+                .bits(ppre3_bits)
+                .hpre()
+                .variant(hpre_bits)
+        });
         // Ensure core prescaler value is valid before future lower
         // core voltage
+        #[cfg(not(feature = "rm0455"))]
         while rcc.d1cfgr.read().d1cpre().bits() != d1cpre_bits {}
+        #[cfg(feature = "rm0455")]
+        while rcc.cdcfgr1.read().cdcpre().bits() != d1cpre_bits {}
 
         // APB1 / APB2 Prescaler
+        #[cfg(not(feature = "rm0455"))]
         rcc.d2cfgr.modify(|_, w| unsafe {
             w.d2ppre1() // D2 contains APB1
                 .bits(ppre1_bits)
                 .d2ppre2() // D2 also contains APB2
                 .bits(ppre2_bits)
         });
+        #[cfg(feature = "rm0455")]
+        rcc.cdcfgr2.modify(|_, w| unsafe {
+            w.cdppre1() // D2/CD contains APB1
+                .bits(ppre1_bits)
+                .cdppre2() // D2/CD also contains APB2
+                .bits(ppre2_bits)
+        });
 
         // APB4 Prescaler
+        #[cfg(not(feature = "rm0455"))]
         rcc.d3cfgr.modify(|_, w| unsafe {
             w.d3ppre() // D3 contains APB4
                 .bits(ppre4_bits)
         });
+        #[cfg(feature = "rm0455")]
+        rcc.srdcfgr.modify(|_, w| unsafe {
+            w.srdppre() // D3 contains APB4
+                .bits(ppre4_bits)
+        });
 
         // Peripheral Clock (per_ck)
         #[cfg(not(feature = "rm0455"))]
@@ -896,23 +962,143 @@ impl Rcc {
         });
         while syscfg.cccsr.read().ready().bit_is_clear() {}
 
+        // This section prints the final register configuration for the main RCC registers:
+        // - System Clock and PLL Source MUX
+        // - PLL configuration
+        // - System Prescalers
+        // Does not include peripheral/MCO/RTC clock MUXes
+        #[cfg(feature = "log")]
+        {
+            debug!("--- RCC register settings");
+
+            let cfgr = rcc.cfgr.read();
+            debug!(
+                "CFGR register: SWS (System Clock Mux)={:?}",
+                cfgr.sws().variant().unwrap()
+            );
+
+            let d1cfgr = rcc.d1cfgr.read();
+            debug!(
+                "D1CFGR register: D1CPRE={:?} HPRE={:?} D1PPRE={:?}",
+                d1cfgr.d1cpre().variant().unwrap(),
+                d1cfgr.hpre().variant().unwrap(),
+                d1cfgr.d1ppre().variant().unwrap(),
+            );
+
+            let d2cfgr = rcc.d2cfgr.read();
+            debug!(
+                "D2CFGR register: D2PPRE1={:?} D2PPRE1={:?}",
+                d2cfgr.d2ppre1().variant().unwrap(),
+                d2cfgr.d2ppre2().variant().unwrap(),
+            );
+
+            let d3cfgr = rcc.d3cfgr.read();
+            debug!(
+                "D3CFGR register: D3PPRE={:?}",
+                d3cfgr.d3ppre().variant().unwrap(),
+            );
+
+            let pllckselr = rcc.pllckselr.read();
+            debug!(
+                "PLLCKSELR register: PLLSRC={:?} DIVM1={:#x} DIVM2={:#x} DIVM3={:#x}",
+                pllckselr.pllsrc().variant(),
+                pllckselr.divm1().bits(),
+                pllckselr.divm2().bits(),
+                pllckselr.divm3().bits(),
+            );
+
+            let pllcfgr = rcc.pllcfgr.read();
+            debug!(
+                "PLLCKSELR register (PLL1): PLL1FRACEN={:?} PLL1VCOSEL={:?} PLL1RGE={:?} DIVP1EN={:?} DIVQ1EN={:?} DIVR1EN={:?}",
+                pllcfgr.pll1fracen().variant(),
+                pllcfgr.pll1vcosel().variant(),
+                pllcfgr.pll1rge().variant(),
+                pllcfgr.divp1en().variant(),
+                pllcfgr.divq1en().variant(),
+                pllcfgr.divr1en().variant(),
+            );
+            debug!(
+                "PLLCKSELR register (PLL2): PLL2FRACEN={:?} PLL2VCOSEL={:?} PLL2RGE={:?} DIVP2EN={:?} DIVQ2EN={:?} DIVR2EN={:?}",
+                pllcfgr.pll2fracen().variant(),
+                pllcfgr.pll2vcosel().variant(),
+                pllcfgr.pll2rge().variant(),
+                pllcfgr.divp2en().variant(),
+                pllcfgr.divq2en().variant(),
+                pllcfgr.divr2en().variant(),
+            );
+            debug!(
+                "PLLCKSELR register (PLL3): PLL3FRACEN={:?} PLL3VCOSEL={:?} PLL3RGE={:?} DIVP3EN={:?} DIVQ3EN={:?} DIVR3EN={:?}",
+                pllcfgr.pll3fracen().variant(),
+                pllcfgr.pll3vcosel().variant(),
+                pllcfgr.pll3rge().variant(),
+                pllcfgr.divp3en().variant(),
+                pllcfgr.divq3en().variant(),
+                pllcfgr.divr3en().variant(),
+            );
+
+            let pll1divr = rcc.pll1divr.read();
+            debug!(
+                "PLL1DIVR register: DIVN1={:#x} DIVP1={:#x} DIVQ1={:#x} DIVR1={:#x}",
+                pll1divr.divn1().bits(),
+                pll1divr.divp1().bits(),
+                pll1divr.divq1().bits(),
+                pll1divr.divr1().bits(),
+            );
+
+            let pll1fracr = rcc.pll1fracr.read();
+            debug!(
+                "PLL1FRACR register: FRACN1={:#x}",
+                pll1fracr.fracn1().bits(),
+            );
+
+            let pll2divr = rcc.pll2divr.read();
+            debug!(
+                "PLL2DIVR register: DIVN2={:#x} DIVP2={:#x} DIVQ2={:#x} DIVR2={:#x}",
+                pll2divr.divn2().bits(),
+                pll2divr.divp2().bits(),
+                pll2divr.divq2().bits(),
+                pll2divr.divr2().bits(),
+            );
+
+            let pll2fracr = rcc.pll2fracr.read();
+            debug!(
+                "PLL2FRACR register: FRACN2={:#x}",
+                pll2fracr.fracn2().bits(),
+            );
+
+            let pll3divr = rcc.pll3divr.read();
+            debug!(
+                "PLL3DIVR register: DIVN3={:#x} DIVP3={:#x} DIVQ3={:#x} DIVR3={:#x}",
+                pll3divr.divn3().bits(),
+                pll3divr.divp3().bits(),
+                pll3divr.divq3().bits(),
+                pll3divr.divr3().bits(),
+            );
+
+            let pll3fracr = rcc.pll3fracr.read();
+            debug!(
+                "PLL3FRACR register: FRACN3={:#x}",
+                pll3fracr.fracn3().bits(),
+            );
+        }
+
         // Return frozen clock configuration
         Ccdr {
             clocks: CoreClocks {
-                hclk: Hertz(rcc_hclk),
-                pclk1: Hertz(rcc_pclk1),
-                pclk2: Hertz(rcc_pclk2),
-                pclk3: Hertz(rcc_pclk3),
-                pclk4: Hertz(rcc_pclk4),
+                hclk: Hertz::from_raw(rcc_hclk),
+                pclk1: Hertz::from_raw(rcc_pclk1),
+                pclk2: Hertz::from_raw(rcc_pclk2),
+                pclk3: Hertz::from_raw(rcc_pclk3),
+                pclk4: Hertz::from_raw(rcc_pclk4),
                 ppre1,
                 ppre2,
                 ppre3,
                 ppre4,
-                csi_ck: Some(Hertz(csi)),
-                hsi_ck: Some(Hertz(hsi)),
-                hsi48_ck: Some(Hertz(hsi48)),
-                lsi_ck: Some(Hertz(lsi)),
-                per_ck: Some(Hertz(per_ck)),
+                csi_ck: Some(Hertz::from_raw(csi)),
+                hsi_ck: Some(Hertz::from_raw(hsi)),
+                hsi48_ck: Some(Hertz::from_raw(hsi48)),
+                lsi_ck: Some(Hertz::from_raw(lsi)),
+                per_ck: Some(Hertz::from_raw(per_ck)),
                 hse_ck,
                 mco1_ck,
                 mco2_ck,
@@ -925,10 +1111,10 @@ impl Rcc {
                 pll3_p_ck,
                 pll3_q_ck,
                 pll3_r_ck,
-                timx_ker_ck: Hertz(rcc_timx_ker_ck),
-                timy_ker_ck: Hertz(rcc_timy_ker_ck),
+                timx_ker_ck: Hertz::from_raw(rcc_timx_ker_ck),
+                timy_ker_ck: Hertz::from_raw(rcc_timy_ker_ck),
                 sys_ck,
-                c_ck: Hertz(sys_d1cpre_ck),
+                c_ck: Hertz::from_raw(sys_d1cpre_ck),
             },
             peripheral: unsafe {
                 // unsafe: we consume self which was a singleton, hence
