@@ -1,4 +1,8 @@
 //! Real-Time Clock
+//!
+//! # Examples
+//!
+//! - [RTC example](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/rtc.rs)
 
 use cast::{f32, i32, u16, u32, u8};
 use chrono::prelude::*;
@@ -58,6 +62,7 @@ pub enum RtcClock {
 }
 
 #[derive(Debug, Copy, Clone, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 /// An error preventing the RTC from initializing
 pub enum InitError {
     RtcNotRunning,
@@ -66,6 +71,7 @@ pub enum InitError {
 }
 
 #[derive(Debug, Copy, Clone, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub enum DstError {
     ClockNotInitialized,
     AlreadyDst,
@@ -165,12 +171,12 @@ impl Rtc {
                 assert!(divider < 64, "HSE Divider larger than 63");
                 rcc.cfgr.modify(|_, w| w.rtcpre().bits(divider));
 
-                clocks.hse_ck().map(|x| Hertz(x.0 / u32(divider)))
+                clocks.hse_ck().map(|x| x / u32(divider))
             }
             RtcClock::Lsi => clocks.lsi_ck(),
         }
         .expect("rtc_ker_ck not running")
-        .0;
+        .raw();
 
         assert!(ker_ck <= 1 << 22, "rtc_ker_ck too fast for prescaler");
 
@@ -779,4 +785,14 @@ impl Rtc {
         // We're allowed to change this once after the LSE fails
         self.prec.kernel_clk_mux(backup::RtcClkSel::LSI);
     }
+
+    /// Returns a reference to the inner peripheral
+    pub fn inner(&self) -> &RTC {
+        &self.reg
+    }
+
+    /// Returns a mutable reference to the inner peripheral
+    pub fn inner_mut(&mut self) -> &mut RTC {
+        &mut self.reg
+    }
 }
