@@ -9,38 +9,25 @@ use crate::sai::{GetClkSAI, Sai, SaiChannel, CLEAR_ALL_FLAGS_BITS, INTERFACE};
 use crate::stm32;
 use crate::time::Hertz;
 
-use crate::stm32::{SAI1, SAI2};
-#[cfg(not(feature = "rm0455"))]
-use crate::stm32::{SAI3, SAI4};
-
+use crate::stm32::SAI1;
 #[cfg(feature = "rm0455")]
+use crate::stm32::SAI2;
+#[cfg(feature = "rm0468")]
+use crate::stm32::SAI4;
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
+use crate::stm32::{SAI2, SAI3, SAI4};
+
+#[cfg(any(feature = "rm0455", feature = "rm0468"))]
 use crate::device::sai1::ch::sr;
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 use crate::device::sai4::ch::sr;
 
-#[cfg(feature = "rm0455")]
+#[cfg(any(feature = "rm0455", feature = "rm0468"))]
 type CH = stm32::sai1::CH;
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 type CH = stm32::sai4::CH;
 
-use crate::gpio::gpioa::{PA0, PA1, PA12, PA2};
-use crate::gpio::gpiob::PB2;
-use crate::gpio::gpioc::{PC0, PC1};
-#[cfg(not(feature = "rm0455"))]
-use crate::gpio::gpiod::{
-    PD0, PD1, PD10, PD11, PD12, PD13, PD14, PD15, PD4, PD6, PD8, PD9,
-};
-#[cfg(feature = "rm0455")]
-use crate::gpio::gpiod::{PD11, PD12, PD13, PD6};
-use crate::gpio::gpioe::{
-    PE0, PE11, PE12, PE13, PE14, PE2, PE3, PE4, PE5, PE6,
-};
-use crate::gpio::gpiof::{PF11, PF6, PF7, PF8, PF9};
-use crate::gpio::gpiog::{PG10, PG7, PG9};
-use crate::gpio::gpioh::{PH2, PH3};
-use crate::gpio::gpioi::{PI4, PI5, PI6, PI7};
-use crate::gpio::{Alternate, AF10, AF6, AF8};
-use stm32h7::Variant::Val;
+use crate::gpio::{self, Alternate};
 
 use crate::traits::i2s::FullDuplex;
 // use embedded_hal::i2s::FullDuplex;
@@ -89,6 +76,7 @@ pub enum I2SSync {
 }
 
 #[derive(Copy, Clone, Debug)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub enum I2SError {
     NoChannelAvailable,
 }
@@ -205,18 +193,21 @@ impl I2SChanConfig {
     }
 
     /// Set synchronization type, defaults to Master
+    #[must_use]
     pub fn set_sync_type(mut self, sync_type: I2SSync) -> Self {
         self.sync_type = sync_type;
         self
     }
 
     /// Set the clock strobing edge
+    #[must_use]
     pub fn set_clock_strobe(mut self, clock_strobe: I2SClockStrobe) -> Self {
         self.clock_strobe = clock_strobe;
         self
     }
 
     /// Set the number of slots, this is an advanced configuration.
+    #[must_use]
     pub fn set_slots(mut self, slots: u8) -> Self {
         assert!(slots <= 16);
         self.slots = slots;
@@ -224,6 +215,7 @@ impl I2SChanConfig {
     }
 
     /// Set the offset of the first data bit
+    #[must_use]
     pub fn set_first_bit_offset(mut self, first_bit_offset: u8) -> Self {
         // 5 bits or less
         assert!(first_bit_offset < 0b10_0000);
@@ -232,12 +224,14 @@ impl I2SChanConfig {
     }
 
     ///  Sets when the frame sync is asserted
+    #[must_use]
     pub fn set_frame_sync_before(mut self, frame_sync_before: bool) -> Self {
         self.frame_sync_before = frame_sync_before;
         self
     }
 
     /// Set frame sync to active high, defaults to active low
+    #[must_use]
     pub fn set_frame_sync_active_high(
         mut self,
         frame_sync_active_high: bool,
@@ -249,24 +243,28 @@ impl I2SChanConfig {
     /// Enable oversampling
     ///
     /// Note: the clock frequency must be doubled when enabled
+    #[must_use]
     pub fn set_oversampling(mut self, oversampling: bool) -> Self {
         self.oversampling = oversampling;
         self
     }
 
     /// Disable master clock generator
+    #[must_use]
     pub fn disable_master_clock(mut self) -> Self {
         self.master_clock_disabled = true;
         self
     }
 
     /// Sets the protocol to MSB or LSB
+    #[must_use]
     pub fn set_protocol(mut self, protocol: I2SProtocol) -> Self {
         self.protocol = protocol;
         self
     }
 
     /// Set the mono mode bit, default is to use stereo
+    #[must_use]
     pub fn set_mono_mode(mut self, mono_mode: bool) -> Self {
         self.mono_mode = mono_mode;
         self
@@ -277,6 +275,7 @@ impl I2SChanConfig {
     /// * true - repeat the last values
     ///
     /// Only meaningful in Tx mode when slots <= 2 and mute is enabled
+    #[must_use]
     pub fn set_mute_repeat(mut self, mute_repeat: bool) -> Self {
         if self.dir == I2SDir::Rx || self.slots > 2 {
             panic!("This only has meaning in I2S::Tx mode when slots <= 2");
@@ -290,6 +289,7 @@ impl I2SChanConfig {
     ///
     /// The value set is compared to the number of consecutive mute frames detected in
     /// reception. When the number of mute frames is equal to this value, the Muted even will be generated
+    #[must_use]
     pub fn set_mute_counter(mut self, mute_counter: u8) -> Self {
         if self.dir == I2SDir::Tx {
             panic!("This only has meaning in I2S::Rx mode");
@@ -301,12 +301,14 @@ impl I2SChanConfig {
     }
 
     /// Set the tristate to release the SD output line (HI-Z) at the end of the last data bit
+    #[must_use]
     pub fn set_tristate(mut self, tristate: bool) -> Self {
         self.tristate = tristate;
         self
     }
 
     /// Set the frame size. If None it will be automatically calculated
+    #[must_use]
     pub fn set_frame_size(mut self, frame_size: Option<u8>) -> Self {
         if let Some(frame_size) = frame_size {
             assert!(frame_size >= 8);
@@ -321,37 +323,51 @@ pub struct I2S {
     master: I2SChanConfig,
     slave: Option<I2SChanConfig>,
 }
+
 impl INTERFACE for I2S {}
 
+pub type I2sUsers = I2S;
+
+impl I2sUsers {
+    pub fn new(master: I2SChanConfig) -> Self {
+        Self {
+            master,
+            slave: None,
+        }
+    }
+
+    #[must_use]
+    pub fn add_slave(mut self, slave: I2SChanConfig) -> Self {
+        self.slave.replace(slave);
+        self
+    }
+}
+
 /// Trait to extend SAI peripherals
 pub trait SaiI2sExt<SAI>: Sized {
     type Rec: ResetEnable;
-    fn i2s_ch_a<PINS, T>(
+    fn i2s_ch_a<PINS>(
         self,
         _pins: PINS,
-        audio_freq: T,
+        audio_freq: Hertz,
         data_size: I2SDataSize,
         prec: Self::Rec,
         clocks: &CoreClocks,
-        master: I2SChanConfig,
-        slave: Option<I2SChanConfig>,
+        users: I2sUsers,
     ) -> Sai<SAI, I2S>
     where
-        PINS: I2SPinsChA<Self>,
-        T: Into<Hertz>;
-    fn i2s_ch_b<PINS, T>(
+        PINS: I2SPinsChA<Self>;
+    fn i2s_ch_b<PINS>(
         self,
         _pins: PINS,
-        audio_freq: T,
+        audio_freq: Hertz,
         data_size: I2SDataSize,
         prec: Self::Rec,
         clocks: &CoreClocks,
-        master: I2SChanConfig,
-        slave: Option<I2SChanConfig>,
+        users: I2sUsers,
     ) -> Sai<SAI, I2S>
     where
-        PINS: I2SPinsChB<Self>,
-        T: Into<Hertz>;
+        PINS: I2SPinsChB<Self>;
 }
 
 macro_rules! i2s {
@@ -359,54 +375,48 @@ macro_rules! i2s {
         $(
             impl SaiI2sExt<$SAIX> for $SAIX {
                 type Rec = rec::$Rec;
-                fn i2s_ch_a<PINS, T>(
+                fn i2s_ch_a<PINS>(
                     self,
                     _pins: PINS,
-                    audio_freq: T,
+                    audio_freq: Hertz,
                     data_size: I2SDataSize,
                     prec: rec::$Rec,
                     clocks: &CoreClocks,
-                    master: I2SChanConfig,
-                    slave: Option<I2SChanConfig>,
+                    users: I2sUsers,
                 ) -> Sai<Self, I2S>
                 where
                     PINS: I2SPinsChA<Self>,
-                    T: Into<Hertz>,
                 {
                     Sai::$i2s_saiX_ch_a(
                         self,
                         _pins,
-                        audio_freq.into(),
+                        audio_freq,
                         data_size,
                         prec,
                         clocks,
-                        master,
-                        slave,
+                        users,
                     )
                 }
-                fn i2s_ch_b<PINS, T>(
+                fn i2s_ch_b<PINS>(
                     self,
                     _pins: PINS,
-                    audio_freq: T,
+                    audio_freq: Hertz,
                     data_size: I2SDataSize,
                     prec: rec::$Rec,
                     clocks: &CoreClocks,
-                    master: I2SChanConfig,
-                    slave: Option<I2SChanConfig>,
+                    users: I2sUsers,
                 ) -> Sai<Self, I2S>
                 where
                     PINS: I2SPinsChB<Self>,
-                    T: Into<Hertz>,
                 {
                     Sai::$i2s_saiX_ch_b(
                         self,
                         _pins,
-                        audio_freq.into(),
+                        audio_freq,
                         data_size,
                         prec,
                         clocks,
-                        master,
-                        slave,
+                        users,
                     )
                 }
             }
@@ -419,41 +429,41 @@ macro_rules! i2s {
                     data_size: I2SDataSize,
                     prec: rec::$Rec,
                     clocks: &CoreClocks,
-                    master: I2SChanConfig,
-                    slave: Option<I2SChanConfig>,
+                    users: I2sUsers,
                 ) -> Self
                 where
                     PINS: I2SPinsChA<$SAIX>,
                 {
-                    assert!(master.slots <= NUM_SLOTS);
-                    if let Some(slave) = &slave {
+                    assert!(users.master.slots <= NUM_SLOTS);
+                    if let Some(slave) = &users.slave {
                         assert!(slave.slots <= NUM_SLOTS);
                     }
 
                     // Clock config
-                    let ker_ck_a = $SAIX::sai_a_ker_ck(&prec, clocks)
-                        .expect("SAI kernel clock must run!");
-                    let clock_ratio = if master.oversampling {
+                    let ker_ck_a = $SAIX::sai_a_ker_ck(&prec, clocks);
+                    let clock_ratio = if users.master.oversampling {
                         512
                     } else {
                         256
                     };
                     let mclk_div =
-                        (ker_ck_a.0) / (audio_freq.0 * clock_ratio);
+                        (ker_ck_a) / (audio_freq * clock_ratio);
                     let mclk_div: u8 = mclk_div
                         .try_into()
-                        .expect("SAI kernel clock is out of range for required MCLK");
+                        .expect(concat!(stringify!($SAIX),
+                                        " A: Kernel clock is out of range for required MCLK"
+                        ));
 
                     // Configure SAI peripheral
                     let mut per_sai = Sai {
                         rb: sai,
                         master_channel: SaiChannel::ChannelA,
-                        slave_channel: if slave.is_some() {
+                        slave_channel: if users.slave.is_some() {
                             Some(SaiChannel::ChannelB)
                         } else {
                             None
                         },
-                        interface: I2S { master, slave },
+                        interface: users,
                     };
 
                     per_sai.sai_rcc_init(prec);
@@ -486,41 +496,42 @@ macro_rules! i2s {
                     data_size: I2SDataSize,
                     prec: rec::$Rec,
                     clocks: &CoreClocks,
-                    master: I2SChanConfig,
-                    slave: Option<I2SChanConfig>,
+                    users: I2sUsers,
                 ) -> Self
                 where
                     PINS: I2SPinsChB<$SAIX>,
                 {
-                    assert!(master.slots <= NUM_SLOTS);
-                    if let Some(slave) = &slave {
+                    assert!(users.master.slots <= NUM_SLOTS);
+                    if let Some(slave) = &users.slave {
                         assert!(slave.slots <= NUM_SLOTS);
                     }
 
                     // Clock config
-                    let ker_ck_a = $SAIX::sai_b_ker_ck(&prec, clocks)
-                        .expect("SAI kernel clock must run!");
-                    let clock_ratio = if master.oversampling {
+                    let ker_ck_a = $SAIX::sai_b_ker_ck(&prec, clocks);
+                    let clock_ratio = if users.master.oversampling {
                         512
                     } else {
                         256
                     };
                     let mclk_div =
-                        (ker_ck_a.0) / (audio_freq.0 * clock_ratio);
+                        (ker_ck_a) / (audio_freq * clock_ratio);
                     let mclk_div: u8 = mclk_div
                         .try_into()
-                        .expect("SAI kernel clock is out of range for required MCLK");
+                        .expect(concat!(stringify!($SAIX),
+                                        " B: Kernel clock is out of range for required MCLK"
+                        ));
+
 
                     // Configure SAI peripheral
                     let mut per_sai = Sai {
                         rb: sai,
                         master_channel: SaiChannel::ChannelB,
-                        slave_channel: if slave.is_some() {
+                        slave_channel: if users.slave.is_some() {
                             Some(SaiChannel::ChannelA)
                         } else {
                             None
                         },
-                        interface: I2S { master, slave },
+                        interface: users,
                     };
 
                     per_sai.sai_rcc_init(prec);
@@ -599,14 +610,22 @@ macro_rules! i2s {
 }
 
 i2s! {
-    SAI1, Sai1: [i2s_sai1_ch_a, i2s_sai1_ch_b],
-    SAI2, Sai2: [i2s_sai2_ch_a, i2s_sai2_ch_b]
+    SAI1, Sai1: [i2s_sai1_ch_a, i2s_sai1_ch_b]
 }
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 i2s! {
+    SAI2, Sai2: [i2s_sai2_ch_a, i2s_sai2_ch_b],
     SAI3, Sai3: [i2s_sai3_ch_a, i2s_sai3_ch_b],
     SAI4, Sai4: [i2s_sai4_ch_a, i2s_sai4_ch_b]
 }
+#[cfg(feature = "rm0455")]
+i2s! {
+    SAI2, Sai2: [i2s_sai2_ch_a, i2s_sai2_ch_b]
+}
+#[cfg(feature = "rm0468")]
+i2s! {
+    SAI4, Sai4: [i4s_sai4_ch_a, i4s_sai4_ch_b]
+}
 
 fn i2s_config_channel(
     audio_ch: &CH,
@@ -723,7 +742,7 @@ fn disable_ch(audio_ch: &CH) {
 
 fn read(audio_ch: &CH) -> nb::Result<(u32, u32), I2SError> {
     match audio_ch.sr.read().flvl().variant() {
-        Val(sr::FLVL_A::EMPTY) => Err(nb::Error::WouldBlock),
+        Some(sr::FLVL_A::EMPTY) => Err(nb::Error::WouldBlock),
         _ => Ok((audio_ch.dr.read().bits(), audio_ch.dr.read().bits())),
     }
 }
@@ -736,11 +755,12 @@ fn send(
     // The FIFO is 8 words long. A write consists of 2 words, in stereo mode.
     // Therefore you need to wait for 3/4s to ensure 2 words are available for writing.
     match audio_ch.sr.read().flvl().variant() {
-        Val(sr::FLVL_A::FULL) => Err(nb::Error::WouldBlock),
-        Val(sr::FLVL_A::QUARTER4) => Err(nb::Error::WouldBlock),
+        Some(sr::FLVL_A::FULL) => Err(nb::Error::WouldBlock),
+        Some(sr::FLVL_A::QUARTER4) => Err(nb::Error::WouldBlock),
         _ => {
             unsafe {
-                audio_ch.dr.write(|w| w.bits(left_word).bits(right_word));
+                audio_ch.dr.write(|w| w.bits(left_word));
+                audio_ch.dr.write(|w| w.bits(right_word));
             }
             Ok(())
         }
@@ -792,129 +812,135 @@ macro_rules! pins {
 pins! {
     SAI1:
         MCLK_A: [
-            PE2<Alternate<AF6>>,
-            PG7<Alternate<AF6>>
+            gpio::PE2<Alternate<6>>,
+            gpio::PG7<Alternate<6>>
         ]
         SCK_A: [
-            PE5<Alternate<AF6>>
+            gpio::PE5<Alternate<6>>
         ]
         FS_A: [
-            PE4<Alternate<AF6>>
+            gpio::PE4<Alternate<6>>
         ]
         SD_A: [
-            PB2<Alternate<AF6>>,
-            PC1<Alternate<AF6>>,
-            PD6<Alternate<AF6>>,
-            PE6<Alternate<AF6>>
+            gpio::PB2<Alternate<6>>,
+            gpio::PC1<Alternate<6>>,
+            gpio::PD6<Alternate<6>>,
+            gpio::PE6<Alternate<6>>
         ]
         MCLK_B: [
-            PF7<Alternate<AF6>>
+            gpio::PF7<Alternate<6>>
         ]
         SCK_B: [
-            PF8<Alternate<AF6>>
+            gpio::PF8<Alternate<6>>
         ]
         FS_B: [
-            PF9<Alternate<AF6>>
+            gpio::PF9<Alternate<6>>
         ]
         SD_B: [
-            PE3<Alternate<AF6>>,
-            PF6<Alternate<AF6>>
-        ]
-    SAI2:
-        MCLK_A: [
-            PE0<Alternate<AF10>>,
-            PI4<Alternate<AF10>>
-        ]
-        SCK_A: [
-            PD13<Alternate<AF10>>,
-            PI5<Alternate<AF10>>
-        ]
-        FS_A: [
-            PD12<Alternate<AF10>>,
-            PI7<Alternate<AF10>>
-        ]
-        SD_A: [
-            PD11<Alternate<AF10>>,
-            PI6<Alternate<AF10>>
-        ]
-        MCLK_B: [
-            PA1<Alternate<AF10>>,
-            PE6<Alternate<AF10>>,
-            PE14<Alternate<AF10>>,
-            PH3<Alternate<AF10>>
-        ]
-        SCK_B: [
-            PA2<Alternate<AF8>>,
-            PE12<Alternate<AF10>>,
-            PH2<Alternate<AF10>>
-        ]
-        FS_B: [
-            PA12<Alternate<AF8>>,
-            PC0<Alternate<AF8>>,
-            PE13<Alternate<AF10>>,
-            PG9<Alternate<AF10>>
-        ]
-        SD_B: [
-            PA0<Alternate<AF10>>,
-            PE11<Alternate<AF10>>,
-            PF11<Alternate<AF10>>,
-            PG10<Alternate<AF10>>
+            gpio::PE3<Alternate<6>>,
+            gpio::PF6<Alternate<6>>
         ]
 }
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399", feature = "rm0455"))]
+pins! {
+    SAI2:
+        MCLK_A: [
+            gpio::PE0<Alternate<10>>,
+            gpio::PI4<Alternate<10>>
+        ]
+        SCK_A: [
+            gpio::PD13<Alternate<10>>,
+            gpio::PI5<Alternate<10>>
+        ]
+        FS_A: [
+            gpio::PD12<Alternate<10>>,
+            gpio::PI7<Alternate<10>>
+        ]
+        SD_A: [
+            gpio::PD11<Alternate<10>>,
+            gpio::PI6<Alternate<10>>
+        ]
+        MCLK_B: [
+            gpio::PA1<Alternate<10>>,
+            gpio::PE6<Alternate<10>>,
+            gpio::PE14<Alternate<10>>,
+            gpio::PH3<Alternate<10>>
+        ]
+        SCK_B: [
+            gpio::PA2<Alternate<8>>,
+            gpio::PE12<Alternate<10>>,
+            gpio::PH2<Alternate<10>>
+        ]
+        FS_B: [
+            gpio::PA12<Alternate<8>>,
+            gpio::PC0<Alternate<8>>,
+            gpio::PE13<Alternate<10>>,
+            gpio::PG9<Alternate<10>>
+        ]
+        SD_B: [
+            gpio::PA0<Alternate<10>>,
+            gpio::PE11<Alternate<10>>,
+            gpio::PF11<Alternate<10>>,
+            gpio::PG10<Alternate<10>>
+        ]
+}
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 pins! {
     SAI3:
         MCLK_A: [
-            PD15<Alternate<AF6>>
+            gpio::PD15<Alternate<6>>
         ]
         SCK_A: [
-            PD0<Alternate<AF6>>
+            gpio::PD0<Alternate<6>>
         ]
         FS_A: [
-            PD4<Alternate<AF6>>
+            gpio::PD4<Alternate<6>>
         ]
         SD_A: [
-            PD1<Alternate<AF6>>
+            gpio::PD1<Alternate<6>>
         ]
         MCLK_B: [
-            PD14<Alternate<AF6>>
+            gpio::PD14<Alternate<6>>
         ]
         SCK_B: [
-            PD8<Alternate<AF6>>
+            gpio::PD8<Alternate<6>>
         ]
         FS_B: [
-            PD10<Alternate<AF6>>
+            gpio::PD10<Alternate<6>>
         ]
         SD_B: [
-            PD9<Alternate<AF6>>
+            gpio::PD9<Alternate<6>>
         ]
+}
+#[cfg(any(feature = "rm0433", feature = "rm0399", feature = "rm0468"))]
+pins! {
     SAI4:
         MCLK_A: [
-            PE2<Alternate<AF8>>
+            gpio::PE2<Alternate<8>>
         ]
         SCK_A: [
-            PE5<Alternate<AF8>>
+            gpio::PE5<Alternate<8>>
         ]
         FS_A: [
-            PE4<Alternate<AF8>>
+            gpio::PE4<Alternate<8>>
         ]
         SD_A: [
-            PB2<Alternate<AF8>>,
-            PC1<Alternate<AF8>>,
-            PD6<Alternate<AF8>>,
-            PE6<Alternate<AF8>>
+            gpio::PB2<Alternate<8>>,
+            gpio::PC1<Alternate<8>>,
+            gpio::PD6<Alternate<8>>,
+            gpio::PE6<Alternate<8>>
         ]
         MCLK_B: [
-            PF7<Alternate<AF8>>
+            gpio::PF7<Alternate<8>>
         ]
         SCK_B: [
-            PF8<Alternate<AF8>>
+            gpio::PF8<Alternate<8>>
         ]
         FS_B: [
-            PF9<Alternate<AF8>>
+            gpio::PF9<Alternate<8>>
         ]
         SD_B: [
-            PE3<Alternate<AF8>>,
-            PF6<Alternate<AF8>>
+            gpio::PE3<Alternate<8>>,
+            gpio::PF6<Alternate<8>>
         ]
 }
